#!/usr/bin/env node
'use strict';

var hydeConfig = {
  observables: {}
};

var npm = require('npm');
var path = require('path');


/**
 * depthFirstTreverse - Treverse tree in depth first fashion
 *
 * @param  {type} startNode     This is where to start the search
 * @param  {type} checkChildren This is a function with the current node as the
 *                              first paramture and the second is the list to
 *                              add too for checking if children are found
 * @return {type}               Returns an array of all the nodes found in depths
 *                              first fashion.
 */
function depthFirstTreverse(startNode, checkChildren) {
  var checking = [startNode];
  var checked = [];

  while (checking.length > 0) {
    var current = checking.shift();

    checkChildren(current, checking);

    checked.unshift(current);
  }

  return checked;
}

npm.load(function(err, npm) {
  npm.commands.ls([], true, function(err, data, lite) {
    depthFirstTreverse(data, function(node, checking) {
      Object.keys(node.dependencies).forEach(function (key) {
        checking.push(node.dependencies[key]);
      });
    }).filter(function (node){
      return node.name.toLowerCase().indexOf('hyde-') === 0;
    }).map(function (node) {
      var loading = require(path.join(node.path, node.main));
      var initData = loading.init();
      initData.observables.forEach(function (observable) {
        hydeConfig.observables[node.name + '@' + node.version +
          ':' + observable.name] = observable.observable;
      });
    });

    console.log(hydeConfig);
  });
});
